#ifndef __BOOST__IO__FORMATLIST__HPP
#define __BOOST__IO__FORMATLIST__HPP
#  include <boost/config.hpp>

#  define FORMATLIST__VERSION                    0x0111

#  if defined(BOOST_MSVC)
#     pragma warning( push )
#     pragma warning( disable : 4227 ) // anachronism used : qualifiers on reference are ignored
#  endif

#  include "format_traits.hpp"
#  include "formatter.hpp"
#  include "format_objects.hpp"
#  include "formatlist_t.hpp"

   namespace boost { namespace io
   {
      // basic format manipulator:
      // NOTE: If you want a wchar_t formatter, you must use formatlistex or a formatter object.

      template< typename ForwardIterator >
      inline formatlist_t< ForwardIterator >     formatlist
                                                 (
                                                    ForwardIterator,
                                                    ForwardIterator
                                                 );

      template< typename Container >
      inline formatlist_t< typename Container::iterator >
                                                 formatlist
                                                 (
                                                    Container &
                                                 );

      // advanced format manipulator:

      template< typename ForwardIterator, typename FormatType, class RefType >
      inline formatlist_t< ForwardIterator, FormatType >
                                                 formatlist
                                                 (
                                                    ForwardIterator,
                                                    ForwardIterator,
                                                    const formatter_t< FormatType, RefType > & const
                                                 );

      template< typename Container, typename FormatType, class RefType >
      inline formatlist_t< typename Container::iterator, FormatType >
                                                 formatlist
                                                 (
                                                    Container &,
                                                    const formatter_t< FormatType, RefType > & const
                                                 );

      // extended format manipulator:

      template< typename FormatType, typename ForwardIterator >
      inline formatlist_t< ForwardIterator, FormatType >
                                                 formatlistex
                                                 (
                                                    ForwardIterator,
                                                    ForwardIterator
                                                 );

      template< typename FormatType, typename Container >
      inline formatlist_t< typename Container::iterator, FormatType >
                                                 formatlistex
                                                 (
                                                    Container & 
                                                 );

      // format manipulator -- output functor versions:

      template< typename ForwardIterator, class Outputter >
      inline formatlist_t< ForwardIterator, typename Outputter::listfmt_type, Outputter >
                                                 formatlistfn
                                                 (
                                                    ForwardIterator,
                                                    ForwardIterator,
                                                    Outputter
                                                 );
      
      template< typename Container, class Outputter >
      inline formatlist_t< typename Container::iterator, typename Outputter::listfmt_type, Outputter >
                                                 formatlistfn
                                                 (
                                                    Container &,
                                                    Outputter
                                                 );
   }}

#  include "formatlist.ipp"

#  if defined(BOOST_MSVC)
#     pragma warning( pop )
#  endif
#endif
