#include <iostream>  // std::cout
#include <fstream>   // std::ofstream
#include <string>    // std::srting

#include <algorithm> // std::generate
#include <vector>    // std::vector
#include <list>      // std::list
#include <map>       // std::map

#include <boost/io/formatlist.hpp> // boost::io::formatlist

// Using extended characters such as the mathematical symbols (\x2200 - \22FF) causes the
// output to halt in the wide file stream.
// *  Is this a bug in my code or the MSVC library?

// tested on: MSVC 7 using the supplied standard library; WinXP; Pentium II

#define EXTENDED_CHARACTER_BUG

int __cdecl main()
{
   // std::vector

   std::vector< int >                  v( 10 );
   std::generate( v.begin(), v.end(), std::rand );

   // std::list

   std::list< char >                   l;
   l.push_back( 'A' );
   l.push_back( 'B' );
   l.push_back( 'C' );

#  if (FORMATLIST__VERSION >= 0x0104) // new examples 04/04/2003:
      std::list< std::wstring >        wl;
      wl.push_back( L"Solange ich Dich fhle," );
      wl.push_back( L"Solange ich Dich spr'," );
      wl.push_back( L"Solange ich in mir whle," );
      wl.push_back( L"Solange," );
      wl.push_back( L"Solang glaub' ich, ich fhr Gutes im Sinn." );
#  endif

   // std::string and arrays

   const char                        * str  = "Jeanette Biedermann";
   std::string                         s    = "Warum bist du nicht da?";

   int                                 i[ 4 ] = { 3, 6, 9, 12 };

#  if (FORMATLIST__VERSION >= 0x0104) // new examples 04/04/2003:
      std::wofstream                   wout( "woutput.txt" );

      const wchar_t                  * wstr = L"S\xF6hne Mannheim";
      std::wstring                     ws   = L"Dein Glck Liegt Mir Am Herzen";
#  endif

   // basic output examples
   // NOTE: Because 2D arrays are not supported, the 2D example does not work as intended.

   std::cout << "basic output:" << '\n' << '\n';

   std::cout << "vector< int > = "         << boost::io::formatlist( v ) << '\n';
   std::cout << "list< char > = "          << boost::io::formatlist( l ) << '\n';
   std::cout << "int[ 4 ] = "              << boost::io::formatlist( i, i + 4 ) << '\n';
   std::cout << "char * = "                << boost::io::formatlist( str, str + std::strlen( str )) << '\n';
   std::cout << "std::string = "           << boost::io::formatlist( s ) << '\n';

#  if (FORMATLIST__VERSION >= 0x0104) // new examples 04/04/2003:
      std::cout  << "std::string = " << boost::io::formatlistex< const char * >( s ) << '\n';

      wout << L"basic output:\n\n";

      wout << L"wchar_t * = "
           << boost::io::formatlistex< wchar_t >( wstr, wstr + wcslen( wstr )) << L'\n';
      wout << L"std::wstring = "
           << boost::io::formatlistex< wchar_t >( ws ) << L'\n';
#  endif

   // list format control

   std::cout << '\n' << "formatting options:" << '\n' << '\n';

   std::cout << "vector< int > = " << boost::io::formatlist( v ).format( '(', ')' ) << '\n';
   std::cout << "vector< int > = " << boost::io::formatlist( v ).format( '|' ).space( true, true ) << '\n';

   std::cout << "int[ 4 ] = "
             << boost::io::formatlist( i, i + 4 ).format( ';' ).space( true, true )
             << '\n';

   std::cout << "char * = "
             << boost::io::formatlist( str, str + std::strlen( str )).format( ';' ).space( false )
             << '\n';

   std::cout << "std::string = "
             << boost::io::formatlist( s ).format( '\0', '\0', '|' ).space( false, false, false )
             << '\n';

   std::cout << '\n' << "sub-list:" << '\n' << '\n';

   std::cout << "std::string = "
             << boost::io::formatlist( s.begin() + 6, s.end()).space( false, false, false ).format( '"', '"', '-' )
             << '\n';

#  if (FORMATLIST__VERSION >= 0x0104) // new examples 04/04/2003:
      std::cout << '\n' << "advanced formatting:" << '\n' << '\n';

      std::cout << "std::string = "
                << boost::io::formatlistex< const char * >( s ).format( "<:", ":>", "\0" ) << '\n';

      std::cout << "std::string = "
                << boost::io::formatlistex< const char * >( s )
                   .format( "{ '", "' }", "', '" )
                   .space( false, false, false )
                << '\n';

      std::cout << "int[ 4 ] = "
                << boost::io::formatlistex< const char * >( i, i + 4 ).format( "-:-" ).space( true, true ) << '\n';

      std::cout << "int[ 4 ] = "
                << boost::io::formatlistex< const char * >( i, i + 4 )
                   .format( "-\n-" ).format( "\n-", "-" )
                   .space( true, true )
                << '\n';

      wout << L'\n' << L"advanced formatting:" << L'\n' << L'\n';

      wout << L"std::wstring = "
           << boost::io::formatlistex< wchar_t >( ws )
#          if !defined(EXTENDED_CHARACTER_BUG)
              .format( L'\x2202' ).space( true, false )     // partial differential
              .format( L'\x3008', L'\x3009' ).space( true ) // left and right angle brackets
#          else
              .format( L'<', L'>' ).space( true ) // left and right angle brackets (ascii versions)
#          endif
           << L'\n';

      wout << L"std::wstring = "
           << boost::io::formatlistex< wchar_t >( ws )
#          if !defined(EXTENDED_CHARACTER_BUG)
              .format( L'\x5FC3' ).space( true, true ) // chinese character: heart
#          else
              .format( L'+' ).space( true, true )
#          endif
           << L'\n';

      wout << L"std::list< std::wstring > = "
           << boost::io::formatlistex< wchar_t * >( wl )
#          if !defined(EXTENDED_CHARACTER_BUG)
              .format( L"\n  ", L"\0", L"\n\x21D2 " ).space( false, false, false ) // implies (rightwards double arrow)
#          else
              .format( L"\n    ", L"\0", L"\n==> " ).space( false, false, false )
#          endif
           << L'\n';
#  endif

#  if (FORMATLIST__VERSION >= 0x0106) // new examples 06/04/2003:
      std::cout << '\n' << "advanced format configuration:" << '\n';

      boost::io::formatter< char >     fmt;
      fmt.format( '(', ')' );
      fmt.space(  true, true );

      boost::io::formatter< char * >   fmt2;
      fmt2.format( "--" );
      fmt2.space(  true, true );
      
      // Mechanism #1: pass a formatter object to the formatlist object:

      std::cout << '\n' << "mechanism 1:" << '\n';

      std::cout << "int[ 4 ] = " << boost::io::formatlist( i, i + 4 ).format( fmt ) << '\n';
      std::cout << "char * = "   << boost::io::formatlist( str, str + std::strlen( str )).format( fmt ) << '\n';

      std::cout << "int[ 4 ] = " << boost::io::formatlist( l ).format(  fmt ) << '\n'; // format  only
      std::cout << "int[ 4 ] = " << boost::io::formatlist( l ).space(   fmt ) << '\n'; // spacing only
      std::cout << "int[ 4 ] = " << boost::io::formatlist( l ).inherit( fmt ) << '\n'; // format *and* spacing

      // Mechanism #2: pass a formatter object to the formatlist manipulator:

      std::cout << '\n' << "mechanism 1 and 2 side-by-side:" << '\n';

      // NOTE: These two are equivalent, but the second will be more efficient
      // and has less typing

      std::cout << "vector< int > = " << boost::io::formatlist( v ).inherit( fmt ) << '\n';
      std::cout << "vector< int > = " << boost::io::formatlist( v, fmt ) << '\n';

      // These two are also equivalent:
      // NOTE: The second version is shorter since it gets the format type from
      // the formatter object

      std::cout << "int[ 4 ] = "      << boost::io::formatlistex< char * >( i, i + 4 ).inherit( fmt2 ) << '\n';
      std::cout << "int[ 4 ] = "      << boost::io::formatlist( i, i + 4, fmt2 ) << '\n';
#  endif

   // The following require more code to provide basic outputting

#  if (FORMATLIST__VERSION >= 0x0107) // new examples 06/04/2003:
      std::map< std::string, std::string >
                                       m;
      m[ "Abscheid Nehmen"    ] = "Farewell";
      m[ "Engel"              ] = "Angel";
      m[ "Mas Alla"           ] = "Beyond";
      m[ "Wenn Das Liebe Ist" ] = "If This Is Love";

      std::cout << '\n' << "std::pair output:" << '\n' << '\n';

      // now supports std::pair formatting 

      boost::io::stdpair_output< char >
                                       pairfmt1;
      pairfmt1.format( '(', ')' );

      boost::io::stdpair_output< char * >
                                       pairfmt2;
      pairfmt2.format( "(", ")", "|" ).space( true, true );

      boost::io::stdpair_output< char * >
                                       pairfmt3;
      pairfmt3.format( "\0", "\0", "==>" ).space( true, true, false );

      std::cout << "map< string, string > = " << boost::io::formatlistfn( m, pairfmt1 ) << '\n';

      std::cout << "map< string, string > = "
                << boost::io::formatlistfn( m, pairfmt1 )
                   .format( '\n', '\0', '\n' ).space( false, false, false )
                << '\n' << '\n';

      std::cout << "map< string, string > = "
                << boost::io::formatlistfn( m, pairfmt2 )
                   .format( "\n==>", "\0", "\n==>" ).space( false, true, true )
                << '\n' << '\n';

      std::cout << "map< string, string > = "
                << boost::io::formatlistfn( m, pairfmt3 )
                   .format( "\n   ", "\0", "\n   " ).space( false, false, false )
                << '\n';
#  endif

#  if (FORMATLIST__VERSION >= 0x0108) // new examples 06/04/2003:
      int                              a2D[ 3 ][ 3 ] =
      {
         { 1, 0, 0 },
         { 0, 1, 0 },
         { 0, 0, 1 }
      };

      std::list< std::list< char > >   tictactoe;
      tictactoe.push_back( std::list< char >());
         tictactoe.back().push_back( 'O' );
         tictactoe.back().push_back( 'X' );
         tictactoe.back().push_back( 'O' );
      tictactoe.push_back( std::list< char >());
         tictactoe.back().push_back( 'X' );
         tictactoe.back().push_back( 'O' );
         tictactoe.back().push_back( 'X' );
      tictactoe.push_back( std::list< char >());
         tictactoe.back().push_back( 'O' );
         tictactoe.back().push_back( 'X' );
         tictactoe.back().push_back( 'O' );

      // now supports 2D arrays and containers (including mixed array/container types) 

      std::cout << '\n' << "2D arrays:" << '\n' << '\n';

      boost::io::array_output< char >
                                       arrayfmt;
      arrayfmt.format( '(', ')' );

      boost::io::container_output< char >
                                       containerfmt;
      containerfmt.format( '|', '|', '|' ).space( true, true );

      std::cout << "int[ 3 ][ 3 ] = " << boost::io::formatlistfn( a2D, a2D + 3, arrayfmt( 3 )) << '\n';
      
      std::cout << "int[ 3 ][ 3 ] = "
                << boost::io::formatlistfn( a2D, a2D + 3, arrayfmt( 3 ))
                   .format( '\n', '\0', '\n' ).space( false, false, false )
                << '\n' << '\n';

      std::cout << "int[ 3 ][ 3 ] = "
                << boost::io::formatlistfn( a2D, a2D + 3, arrayfmt( 1, 2 ))
                   .format( '\n', '\0', '\n' ).space( false, false, false )
                << '\n' << '\n';

      std::cout << "std::list< std::list< char > > tictactoe = "
                << boost::io::formatlistfn( tictactoe, containerfmt )
                   .format( '\n', '\0', '\n' ).space( false, false, false )
                << '\n';
#  endif

#  if (FORMATLIST__VERSION >= 0x0109) // new examples 06/04/2003:
      char                             a3D[ 3 ][ 3 ][ 3 ] =
      {
         {
            { 'A', '0', '0' },
            { '0', 'A', '0' },
            { '0', '0', 'A' }
         }, {
            { 'B', '0', '0' },
            { '0', 'B', '0' },
            { '0', '0', 'B' }
         }, {
            { 'C', '0', '0' },
            { '0', 'C', '0' },
            { '0', '0', 'C' }
         },
      };

      // Now supports nD arrays and containers via nested output functors:

      std::cout << '\n' << "3D arrays:" << '\n' << '\n';

      // NOTE: the reuse of arrayfmt for the 2D array rendering.

      boost::io::array_output< char, boost::io::array_output< char > >
                                       array3Dfmt( arrayfmt( 3 ));
      array3Dfmt.format( '{', '}', '|' ).space( true, true, true );

      std::cout << "char[ 3 ][ 3 ][ 3 ] = "
                << boost::io::formatlistfn( a3D, a3D + 3, array3Dfmt( 3 ))
                   .format( '\n', '\0', '\n' ).space( false, false, false )
                << '\n' << '\n';
#  endif

   return( 0 );
}
